@@ -1,4 +1,4 @@
-/* $Header: /u/christos/src/tcsh-6.06/RCS/tw.parse.c,v 3.73 1995/04/16 19:15:53 christos Exp $ */
+/* $Header: /u/christos/cvsroot/tcsh/tw.parse.c,v 3.76 1996/10/05 17:39:20 christos Exp $ */
 /*
  * tw.parse.c: Everyone has taken a shot in this futile effort to
  *	       lexically analyze a csh line... Well we cannot good
@@ -39,7 +39,7 @@
  */
 #include "sh.h"
 
-RCSID("$Id: tw.parse.c,v 3.73 1995/04/16 19:15:53 christos Exp $")
+RCSID("$Id: tw.parse.c,v 3.76 1996/10/05 17:39:20 christos Exp $")
 
 #include "tw.h"
 #include "ed.h"
@@ -105,7 +105,6 @@ extern int lbuffed;		/* from sh.print.c */
 
 static	void	 extract_dir_and_name	__P((Char *, Char *, Char *));
 static	int	 insert_meta		__P((Char *, Char *, Char *, bool));
-static	Char	*dollar			__P((Char *, Char *));
 static	Char	*tilde			__P((Char *, Char *));
 static  int      expand_dir		__P((Char *, Char *, DIR  **, COMMAND));
 static	bool	 nostat			__P((Char *));
@@ -408,7 +407,7 @@ tenematch(inputline, num_read, command)
 	return search_ret;
 
     default:
-	xprintf(CGETS(30, 4, "tcsh: Internal match error.\n"));
+	xprintf(CGETS(30, 4, "%s: Internal match error.\n"), progname);
 	return 1;
 
     }
@@ -1605,7 +1604,8 @@ t_search(word, wp, command, max_word_length, looking, list_max, pat, suf)
 
     default:
 	xprintf(CGETS(30, 9,
-		"\ntcsh internal error: I don't know what I'm looking for!\n"));
+		"\n%s internal error: I don't know what I'm looking for!\n"),
+		progname);
 	NeedsRedraw = 1;
 	return (-1);
     }
@@ -1687,7 +1687,7 @@ extract_dir_and_name(path, dir, name)
  * 	expand "/$old1/$old2/old3/"
  * 	to "/value_of_old1/value_of_old2/old3/"
  */
-static Char *
+Char *
 dollar(new, old)
     Char   *new, *old;
 {
@@ -2113,7 +2113,7 @@ fcompare(file1, file2)
 void
 catn(des, src, count)
     register Char *des, *src;
-    register count;
+    int count;
 {
     while (--count >= 0 && *des)
 	des++;
@@ -2131,7 +2131,7 @@ catn(des, src, count)
 void
 copyn(des, src, count)
     register Char *des, *src;
-    register count;
+    int count;
 {
     while (--count >= 0)
 	if ((*des++ = *src++) == 0)
@@ -2152,10 +2152,15 @@ tgetenv(str)
     int     len, res;
 
     len = (int) Strlen(str);
+    /* Search the STR_environ for the entry matching str. */
     for (var = STR_environ; var != NULL && *var != NULL; var++)
-	if ((*var)[len] == '=') {
+	if (Strlen(*var) >= len && (*var)[len] == '=') {
+	  /* Temporarily terminate the string so we can copy the variable
+	     name. */
 	    (*var)[len] = '\0';
 	    res = StrQcmp(*var, str);
+	    /* Restore the '=' and return a pointer to the value of the
+	       environment variable. */
 	    (*var)[len] = '=';
 	    if (res == 0)
 		return (&((*var)[len + 1]));
